/*
 * Copyright (C) 2005 Universitat d'Alacant / Universidad de Alicante
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */
	
#include <lttoolbox/PoolVecUshort.H>

PoolVecUshort::PoolVecUshort()
{
  init(1, 1); // at least, for better performance
}

PoolVecUshort::~PoolVecUshort()
{
  destroy();
}

PoolVecUshort::PoolVecUshort(unsigned int nelems, unsigned int size)
{
  init(nelems, size);
}

PoolVecUshort::PoolVecUshort(PoolVecUshort const &p)
{
  copy(p);
}

void
PoolVecUshort::copy(PoolVecUshort const &p)
{
  created = p.created;

  // all new members are available
  for(list<vector<unsigned short> >::iterator it = created.begin();
      it != created.end(); it++)
  {
    free.push_back(&(*it));
  }
}

void
PoolVecUshort::destroy()
{
}

void
PoolVecUshort::init(unsigned int nelems, unsigned int size)
{
  created.clear();
  sizevec = size;

  vector<unsigned short> tmp(sizevec);  
  for(unsigned int i = 0; i < nelems; i++)
  {
    created.push_front(tmp);
  }

  free.clear();
  for(list<vector<unsigned short> >::iterator it = created.begin();
      it != created.end(); it++)
  {
    free.push_front(&(*it));
  }
}

vector<unsigned short> *
PoolVecUshort::get()
{
  vector<unsigned short> *result;

  if(free.size() == 0)
  {
    vector<unsigned short> new_v(sizevec);
    created.push_front(new_v);
    free.push_front(&(*(created.begin())));
  }

  result = *(free.begin());
  free.erase(free.begin());

  return result;
}

void
PoolVecUshort::release(vector<unsigned short> *item)
{
  free.push_front(item);
}
