/*
 * Copyright (C) 2005 Universitat d'Alacant / Universidad de Alicante
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */
	
#ifndef _POOL_
#define _POOL_

#include <list>
#include <vector>

using namespace std;

class PoolVecUshort
{
private:
  /**
   * List of free vecs
   */
  list<vector<unsigned short> *> free;

  /**
   * List of vecs
   */
  list<vector<unsigned short> > created;

  /**
   * Default size
   */
  unsigned int sizevec;
  
  /**
   * Copy method
   * @param p the pool to be replicated
   */
  void copy(PoolVecUshort const &p);
 
  /**
   * Destroy method
   */
  void destroy();

public:

  /**
   * Constructor
   */
  PoolVecUshort();

  /**
   * Parametric constructor
   * @param nelems number of vecs to create initially
   * @param size size of that elements (all the same)
   */
  PoolVecUshort(unsigned int nelems, unsigned int size);

  /** 
   * Copy constructor
   * @param p the pool to be copied
   */
  PoolVecUshort(PoolVecUshort const &p);

  /**
   * Destructor
   */
  ~PoolVecUshort();
  
  /**
   * Initialization function, works as the parametric constructor
   * @param nelems number of vecs to create initially
   * @param size size of that elements (all the same)
   */
  void init(unsigned int nelems, unsigned int size);

  /**
   * Release method, frees a vector to be reused
   * @param item the vector
   */
  void release(vector<unsigned short> *item);

  /**
   * Get method
   * @return the vector
   */
  vector<unsigned short> * get();
};


#endif
